# -*- coding: utf-8 -*-
"""@author: skynet-devel."""

from pathlib import Path


class DeviceItem:
    """DeviceItem."""

    def __init__(self):
        self.__object_path = ""
        self.__path = None
        self.__vendor = "[No Vendor]"
        self.__model = "[No Model]"
        self.__size = 0

    @property
    def object_path(self) -> str:
        """Getter."""
        return self.__object_path

    @object_path.setter
    def object_path(self, value: str) -> None:
        """Setter."""
        self.__object_path = value

    @property
    def path(self) -> Path:
        """Getter."""
        return self.__path

    @path.setter
    def path(self, value: str) -> Path:
        """Setter."""
        self.__path = value

    @property
    def vendor(self) -> str:
        """Getter."""
        return self.__vendor

    @vendor.setter
    def vendor(self, value: str) -> None:
        """Setter."""
        self.__vendor = value

    @property
    def model(self) -> str:
        """Getter."""
        return self.__model

    @model.setter
    def model(self, value: str) -> None:
        """Setter."""
        self.__model = value

    @property
    def size(self) -> int:
        """Getter."""
        return self.__size

    @size.setter
    def size(self, value: int) -> None:
        """Setter."""
        self.__size = value



# -*- coding: utf-8 -*-
"""Threading."""

import os
from pathlib import Path
from typing import Union

from gi.repository import GLib, Gio
from PySide6 import QtCore
from PySide6.QtCore import QObject, QThread, Slot


MIB = 1024 * 1024


class Signals(QObject):
    """QtCore.Signal."""

    # Application
    cancel_thread = QtCore.Signal()

    # WritingThread
    progressbar_value = QtCore.Signal(int)
    statusbar_message = QtCore.Signal(str)
    result = QtCore.Signal(object)


class WritingThread(QThread):
    """Writing class."""

    def __init__(self,
                 in_file: Path,
                 out_object_path: str,
                 application,
                 blksize: int = MIB):
        super().__init__()

        self.__in_file = in_file
        self.__object_path = out_object_path
        self.__blksize = blksize
        self.__application = application
        self.__udisks2 = application.udisks2

        self.signal = Signals()

        self.__cancel_thread = False

        # SLOTS
        self.__application.signal.cancel_thread.connect(self.cancel)

    @Slot()
    def cancel(self):
        """Cancel thread."""
        self.__cancel_thread = True

    def run(self):
        """Run thread."""
        file_decriptors = self.__get_file_descriptors()
        if not file_decriptors:
            return
        in_fd, out_fd = file_decriptors

        filesize = os.path.getsize(self.__in_file)
        written = 0
        progress_value = 0

        try:
            self.signal.statusbar_message.emit("Writing ...")
            while written < filesize:
                if self.__cancel_thread is True:
                    break

                written += os.sendfile(out_fd, in_fd,
                                       offset=None, count=self.__blksize)
                written_percent = int(written * 100 / filesize)
                if written_percent != progress_value:
                    progress_value = written_percent
                    if progress_value > 99:
                        break
                    self.signal.progressbar_value.emit(progress_value)
        except OSError:
            # Here is not always thrown an exception.
            # (e.g. when removing a USB stick while writing).
            self.__close_all_fd(file_decriptors)
            message = "Error while writing ... Writing canceled."
            self.signal.statusbar_message.emit(message)
        else:
            self.__close_all_fd(file_decriptors)
            if self.__cancel_thread is True:
                self.signal.statusbar_message.emit("Writing canceled.")
            else:
                self.signal.progressbar_value.emit(100)
                self.signal.statusbar_message.emit("Writing finished.")

    def __get_file_descriptors(self) -> tuple:
        file_decriptors = ()

        in_fd = self.__get_fd_image(self.__in_file)
        if in_fd != -1:
            out_fd = self.__get_fd_device(self.__object_path)
            if not out_fd:
                self.__close_all_fd((in_fd,))

        file_decriptors = (in_fd, out_fd)

        return file_decriptors

    def __get_fd_image(self, file: Path) -> int:
        file_fd = -1

        try:
            file_fd = os.open(file, os.O_RDONLY)
        except OSError as error:
            self.signal.statusbar_message.emit(error.args)

        return file_fd

    def __get_fd_device(self, object_path: str) -> Union[int, None]:
        file_fd = None

        interface_block = \
            self.__udisks2.get_interface(object_path, 'Block')

        if not interface_block:
            return None

        try:
            if self.__is_valid_version(min_version=273):
                arg_options = self.__udisks2.preset_args['open_device_restore']
                result = \
                    interface_block.call_open_device_sync('w',
                                                          arg_options,
                                                          Gio.UnixFDList(),
                                                          None)
            else:
                # Deprecated
                arg_options = self.__udisks2.preset_args['open_for_restore']
                result = \
                    interface_block.call_open_for_restore_sync(
                        arg_options,
                        Gio.UnixFDList(),
                        None)
        except GLib.Error as error:
            self.signal.statusbar_message.emit(error.message)
            return file_fd
        else:
            if result:
                out_fd, out_fd_list = result
                file_fd = out_fd_list.get(out_fd.unpack())
            self.__udisks2.settle()

        return file_fd

    def __close_all_fd(self, file_decriptors: tuple) -> bool:
        all_fd_closed = False

        try:
            for file_fd in file_decriptors:
                os.close(file_fd)
        except OSError as error:
            self.signal.statusbar_message.emit(error.args)
        else:
            all_fd_closed = True

        return all_fd_closed

    def __is_valid_version(self, min_version: int) -> bool:
        valid_version = False

        udisks2_version = self.__udisks2.version
        if min_version <= udisks2_version:
            valid_version = True

        return valid_version



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""__main__."""

import sys

from usb_imager.ui_app import Application
from usb_imager.modules.udisks2 import UDisks2
from usb_imager.modules.msgbox import MsgBox


def check_deps() -> None:
    """Check system dependencies."""
    # Check for platform 'linux'
    if not sys.platform.startswith('linux'):
        MsgBox().error("Programm only supports Unix!")
        sys.exit(1)

    # Check for installed UDisks2/D-Bus
    if not UDisks2.has_udisks2():
        MsgBox().error("UDisks2 was not found!")
        sys.exit(1)


def main() -> int:
    """Start application."""
    check_deps()

    app = Application(sys.argv)
    return app.exec()


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
"""@author: Skynet-Devel."""

import os
import re
from pathlib import Path
import importlib.resources
# from typing import Any, Optional

from gi.repository import GLib
from PySide6.QtGui import QIcon
from PySide6.QtCore import QMetaObject, Slot, QSize
from PySide6.QtWidgets import (
    QApplication, QMainWindow, QFileDialog, QMessageBox,
)

from usb_imager.ui_mainwindow import Ui_MainWindow

from usb_imager.appinfo import (
    APP_NAME, APP_VERSION, APP_ORGANISATION, APP_ID, APP_TITLE, APP_ABOUT,
)
from usb_imager.device_item import DeviceItem
from usb_imager.modules.udisks2 import UDisks2
from usb_imager.modules.filesize import FileSize
from usb_imager.threads import Signals, WritingThread


STATUSBAR_TIMEOUT = 5000


class Application(QApplication):
    """Application."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setup_application()
        self.mainwindow = MainWindow()
        self.mainwindow.show()

    def setup_application(self) -> None:
        """Application setup."""
        self.setApplicationName(APP_NAME)
        self.setApplicationVersion(APP_VERSION)
        self.setOrganizationName(APP_ORGANISATION)
        self.setOrganizationDomain(APP_ID)

        path = importlib.resources.files('usb_imager.resources')
        icon = QIcon(str(path) + '/' + 'usb-imager.ico')
        self.setWindowIcon(icon)


class MainWindow(QMainWindow):
    """MainWidget."""

    def __init__(self):
        super().__init__()

        self.init_mainwindow()

        self.signal = Signals()
        self.copythread = None
        # blksize is stored in comboBox_buffers data.

        self.udisks2 = UDisks2()
        self.udisks2.signal_connect_obj_added(self.callback_object_added)
        self.udisks2.signal_connect_obj_removed(self.callback_object_removed)

        self.srcpath = None
        # Target path is stored in comboBox_devices data.

        self.init_target_devices()

    def init_mainwindow(self) -> None:
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.setWindowTitle(APP_TITLE)
        self.setMaximumSize(QSize(16777215, 0))
        QMetaObject.connectSlotsByName(self)

    def init_target_devices(self) -> None:
        block_device_pathes = self.udisks2.get_object_pathes('block_devices')

        for object_path in block_device_pathes:
            if self.is_usb_key(object_path):
                self.add_device_item(object_path)

        sizes = [512, 1024, 2048, 4096, 8192, 16384, 32768, 1024 ** 2]
        for blksize in sizes:
            filesize = str(FileSize(blksize).convert())
            self.ui.comboBox_buffers.addItem(filesize, blksize)

        pos_last = self.ui.comboBox_buffers.count() - 1
        self.ui.comboBox_buffers.setCurrentIndex(pos_last)

    def callback_object_added(self, sender, dbus_object):
        object_path = dbus_object.get_object_path()

        if self.is_usb_key(object_path):
            device_name = self.add_device_item(object_path)
            if device_name:
                self.ui.statusbar.showMessage(f"USB-Device added:  {device_name}",
                                           STATUSBAR_TIMEOUT)

    def callback_object_removed(self, sender, dbus_object):
        object_path = dbus_object.get_object_path()

        device_name = self.remove_device_item(object_path)
        current_msg = self.ui.statusbar.currentMessage()
        if device_name and not current_msg.startswith("Error"):
            self.ui.statusbar.showMessage(f"USB-Device removed:  {device_name}",
                                       STATUSBAR_TIMEOUT)

    def add_device_item(self, object_path: str) -> str:
        device_name = ""

        device_item = DeviceItem()

        drive_path = \
            self.udisks2.get_property(object_path, 'Block', 'drive')

        # Set device_item attributes
        device_item.object_path = object_path
        device_path = self.udisks2.get_property(object_path, 'Block', 'device')
        device_item.path = Path(device_path)

        interface = self.udisks2.get_interface(drive_path, 'Drive')
        device_item.size, vendor, model = \
            interface.get_properties('size', 'vendor', 'model')

        if vendor:
            device_item.vendor = vendor
        if model:
            device_item.model = model

        # Add device_item to combobox
        size = FileSize(device_item.size).convert(max_decimals=1)
        combobox_text = "{} - {} {} - {}".format(
            device_item.path.name, device_item.vendor, device_item.model, size)
        self.ui.comboBox_devices.addItem(combobox_text, device_item)
        device_name = f"{device_item.vendor} {device_item.model}"

        return device_name

    def remove_device_item(self, object_path: str) -> str:
        device_name = ""

        index_count = self.ui.comboBox_devices.count()
        for index in range(index_count):
            device_item = self.ui.comboBox_devices.itemData(index)
            if not device_item:
                continue
            if object_path == device_item.object_path:
                device_name = f"{device_item.vendor} {device_item.model}"
                self.ui.comboBox_devices.removeItem(index)

        return device_name

    @Slot()
    def on_pushButton_open_clicked(self):
        homedir = Path.home() / 'Downloads'
        filters = ('Image Files (*.iso *.img *.raw *.bin *.dd)',)

        filename = QFileDialog.getOpenFileName(
            parent=self,
            caption="Open Image",
            dir=str(homedir),
            filter=';;'.join(filters)
            )[0]

        if filename:
            filename = Path(filename)
            if filename.match('*.iso') and not self.is_bootable_iso(filename):
                return
            self.srcpath = filename
            self.ui.label_srcpath.setText(self.srcpath.name)

    @Slot()
    def on_pushButton_write_clicked(self) -> None:
        if self.copythread and self.copythread.isRunning():
            return

        device_item = self.ui.comboBox_devices.currentData()

        # IS SOURCE IMAGE AND TARGET DEVICE SELECTED?
        if not self.srcpath:
            message = "Please select an image to use!"
            self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)
            return
        if not device_item:
            message = "Please select a device to use!"
            self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)
            return

        # IS THE TARGET DEVICE TOO SMALL FOR WRITING THE IMAGE?
        srcfile_size = os.path.getsize(self.srcpath)
        if srcfile_size > device_item.size:
            message = \
                "Error:  " \
                "The size of the device is too small to write the image."
            self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)
            return

        # ARE ALL PARTITIONS OF THE TARGET DEVICE UNMOUNTED AND UNLOCKED?
        mounted_filesystem_interfaces = \
            self.udisks2.get_mounted_filesystems(device_item.object_path)

        if mounted_filesystem_interfaces:
            # Security question
            text = \
                "This device is mounted in the filesystem.\n\n" \
                "Would you like to unmount it?"
            result = QMessageBox.warning(self, "Warning", text,
                                         QMessageBox.Yes | QMessageBox.No)
            if result == QMessageBox.No:
                return

            for iface_fs in mounted_filesystem_interfaces:
                # Unmount device
                arg_options = self.udisks2.preset_args['unmount']
                try:
                    iface_fs.call_unmount_sync(arg_options)
                except GLib.Error as error:
                    self.ui.statusbar.showMessage(error.message)
                    return

                self.udisks2.settle()

                # Lock device if unlocked
                object_path = iface_fs.get_object_path()
                unlocked_encrypted = \
                    self.udisks2.get_unlocked_encrypted(object_path)
                if unlocked_encrypted:
                    arg_options = self.udisks2.preset_args['lock']
                    try:
                        unlocked_encrypted.call_lock_sync(arg_options)
                    except GLib.Error as error:
                        self.ui.statusbar.showMessage(error.message)
                        return

                    self.udisks2.settle()

        # SECURITY QUESTION BEFORE WRITING.
        text = \
            "The device will be completely overwritten " \
            "and data on it will be lost.\n\n" \
            "Do you want to continue the process?"

        result = QMessageBox.warning(self, "Warning", text,
                                     QMessageBox.Yes | QMessageBox.No)
        if result == QMessageBox.No:
            return

        # START WRITING
        blksize = self.ui.comboBox_buffers.currentData()
        self.copythread = WritingThread(self.srcpath, device_item.object_path,
                                        self, blksize)
        self.start_writing()

    @Slot()
    def on_actionAboutQt_triggered(self) -> None:
        QMessageBox.aboutQt(self)

    @Slot()
    def on_actionAbout_triggered(self) -> None:
        QMessageBox.about(self, "About USB-Imager", APP_ABOUT)

    def is_bootable_iso(self, filepath: Path) -> bool:
        is_bootable = False

        try:
            with filepath.open(mode='rb') as file:
                file.seek(510)
                mbr_signature = file.read(2).hex()
        except OSError as error:
            self.ui.statusbar.showMessage(error.args)
        else:
            if mbr_signature == '55aa':
                is_bootable = True
            else:
                message = "Selected ISO file is not bootable!"
                self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)

        return is_bootable

    def is_usb_key(self, object_path: str) -> bool:
        is_usbdev = True

        whole_block_device_pathes = \
            self.udisks2.get_object_pathes_whole_block_device()
        interface = self.udisks2.get_interface(object_path, 'Block')
        if not interface:
            return False

        if object_path not in whole_block_device_pathes:
            is_usbdev = False
        elif re.search(r"[0-9]$", object_path):
            is_usbdev = False
        else:
            drive_path, device_path, size = \
                interface.get_properties('drive', 'device', 'size')

            if not drive_path or drive_path == '/':
                is_usbdev = False
            elif not device_path or device_path == '/':
                is_usbdev = False
            elif not size or size == 0:
                is_usbdev = False
            else:
                interface = self.udisks2.get_interface(drive_path, 'Drive')
                if not interface:
                    return False

                connection_bus, removable = \
                    interface.get_properties('connection-bus', 'removable')

                if removable is False or connection_bus != "usb":
                    is_usbdev = False

        return is_usbdev

    def start_writing(self):
        # SLOTS
        self.copythread.signal.progressbar_value.connect(
            self.ui.progressBar.setValue)
        self.copythread.signal.statusbar_message.connect(
            self.ui.statusbar.showMessage)
        self.ui.pushButton_cancel.clicked.connect(
            self.signal.cancel_thread.emit)
        self.copythread.finished.connect(
            self.finished_writing)

        self.ui.pushButton_open.setEnabled(False)
        self.ui.comboBox_devices.setEnabled(False)
        self.ui.comboBox_buffers.setEnabled(False)
        self.ui.pushButton_write.setEnabled(False)
        self.ui.statusbar.clearMessage()

        self.copythread.start()

    def finished_writing(self):
        self.copythread.wait()
        self.ui.pushButton_open.setEnabled(True)
        self.ui.comboBox_devices.setEnabled(True)
        self.ui.comboBox_buffers.setEnabled(True)
        self.ui.pushButton_write.setEnabled(True)



# -*- coding: utf-8 -*-
"""
udisks2.

https://lazka.github.io/pgi-docs/UDisks-2.0/

https://lazka.github.io/pgi-docs/UDisks-2.0/classes/Client.html
-> get_object_manager()

https://lazka.github.io/pgi-docs/Gio-2.0/classes/DBusObjectManager.html
-> get_objects()
-> get_object(object_path)
-> get_object_path()
-> get_interface(object_path, interface_name)

interface-added 	Emitted when interface is added to object.
interface-removed 	Emitted when interface has been removed from object.
object-added        Emitted when object is added to manager.
object-removed      Emitted when object is removed from manager.

https://lazka.github.io/pgi-docs/Gio-2.0/classes/DBusObject.html
-> get_interface(interface_name)
-> get_interfaces()
-> get_object_path()

interface-added 	Emitted when interface is added to object.
interface-removed 	Emitted when interface is removed from object.

https://lazka.github.io/pgi-docs/UDisks-2.0/interfaces.html
"""

import os
import sys
from typing import Union, Any

from .msgbox import MsgBox

import gi
gi.require_version('Gio', '2.0')

try:
    gi.require_version('UDisks', '2.0')
except ValueError:
    message = \
    """
    Missing:
        GObject based library to access udisks2 - introspection data.

    Install:
        'gir1.2-udisks-2.0' for debian based linux.
        'typelib-1_0-UDisks-2_0' for openSUSE linux.
    """
    MsgBox().error(message)
    sys.exit(1)

from gi.repository import Gio, UDisks, GLib


_conv_object_path_types = {
    "drives": "/org/freedesktop/UDisks2/drives/",
    "block_devices": "/org/freedesktop/UDisks2/block_devices/",
    "mdraid": "/org/freedesktop/UDisks2/mdraid/",
    "iscsi/session": "/org/freedesktop/UDisks2/iscsi/session/",
    "jobs": "/org/freedesktop/UDisks2/jobs/",
    }

_conv_interface_names = {
    "Manager": "org.freedesktop.UDisks2.Manager",

    "Drive": "org.freedesktop.UDisks2.Drive",
    "Drive.Ata": "org.freedesktop.UDisks2.Drive.Ata",

    "Block": "org.freedesktop.UDisks2.Block",
    "Partition": "org.freedesktop.UDisks2.Partition",
    "PartitionTable": "org.freedesktop.UDisks2.PartitionTable",
    "Filesystem": "org.freedesktop.UDisks2.Filesystem",
    "Encrypted": "org.freedesktop.UDisks2.Encrypted",
    "Loop": "org.freedesktop.UDisks2.Loop",

    "LogicalVolume": "org.freedesktop.UDisks2.LogicalVolume",
    }


class UDisks2:
    """UDisks2 interface."""

    preset_args = \
        {
            'unmount':
                GLib.Variant('a{sv}', {'force': GLib.Variant('b', False)}),
            'lock':
                GLib.Variant('a{sv}',
                             {'auth.no_user_interaction': GLib.Variant(
                                 'b', True)}),
            'open_device_restore':
                GLib.Variant('a{sv}',
                             {'flags': GLib.Variant(
                                 'i', os.O_EXCL | os.O_CLOEXEC | os.O_SYNC)}),
            'open_for_restore':
                GLib.Variant('a{sv}',
                             {'flags': GLib.Variant(
                                 'i', os.O_WRONLY | os.O_SYNC)}),
            }

    def __init__(self):
        self.__client = None
        self.__object_manager = None
        self.version = None

        self.__init()

    def __init(self):
        self.__client = UDisks.Client.new_sync(None)
        self.__object_manager = self.__client.get_object_manager()
        self.version = self.__get_udisks2_version()

    def __get_udisks2_version(self) -> int:
        version = -1

        manager = self.__client.get_manager()
        if manager:
            string = manager.get_property('version').replace('.', '')
            version = int(string)

        return version

    def settle(self):
        """
        Blocks until all pending D-Bus messages have been delivered.

        Also emits the (rate-limited) UDisks.Client ::changed signal
        if changes are currently pending.

        This is useful in two situations: 1. when using synchronous
        method calls since e.g. D-Bus signals received while waiting
        for the reply are queued up and dispatched after the synchronous
        call ends;

        and when using asynchronous calls where the return value references
        a newly created object (such as the Manager.LoopSetup() method).

        Returns
        -------
        None.

        """
        self.__client.settle()

    def signal_connect_obj_added(self, callback_func: Any) -> int:
        handler_id = \
            self.__object_manager.connect("object-added", callback_func)
        return handler_id

    def signal_connect_obj_removed(self, callback_func: Any) -> int:
        handler_id = \
            self.__object_manager.connect("object-removed", callback_func)
        return handler_id

    def signal_handler_block(self, handler_id) -> None:
        self.__object_manager.handler_block(handler_id)

    def signal_handler_unblock(self, handler_id) -> None:
        self.__object_manager.handler_unblock(handler_id)

    def signal_handler_disconnect(self, handler_id) -> None:
        self.__object_manager.handler_disconnect(handler_id)

    @staticmethod
    def has_udisks2() -> bool:
        return bool(UDisks.Client.new_sync(None))

    def get_block_for_drive(self,
                            object_path_drive: str,
                            get_physical: bool) -> Union[UDisks.Block, None]:
        interface_block = None
        object_drive = self.__object_manager.get_object(object_path_drive)

        interface_name = "org.freedesktop.UDisks2.Drive"
        interface_drive = \
            self.__object_manager.get_interface(object_path_drive,
                                                interface_name)
        if object_drive:
            interface_block = \
                self.__client.get_block_for_drive(interface_drive,
                                                  get_physical)
        return interface_block

    def get_object(self, object_path: str) -> Union[Gio.DBusObject, None]:
        dbus_object = self.__object_manager.get_object(object_path)
        return dbus_object

    def get_objects(self, object_type: str = None) -> list[Gio.DBusObject]:
        objects = []

        objects_all = self.__object_manager.get_objects()

        if object_type:
            if object_type in _conv_object_path_types:
                string = _conv_object_path_types[object_type]
            else:
                valid_pathtypes = tuple(_conv_object_path_types)
                error_message = \
                    f"Valid object_type arguments are {valid_pathtypes}."
                raise ValueError(error_message)
        else:
            string = ""

        for object_ in objects_all:
            object_path = object_.get_object_path()
            if object_path.startswith(string):
                objects.append(object_)

        return objects

    def get_object_pathes(self, object_type: str = None) -> list[str]:
        object_pathes = []

        objects = self.get_objects(object_type)
        if objects:
            object_pathes = sorted([obj.get_object_path()
                                    for obj in objects])
        return object_pathes

    def get_object_pathes_whole_block_device(self):
        object_pathes_whole_disk = []

        object_pathes = self.get_object_pathes("drives")
        for object_path in object_pathes:
            interface = \
                self.get_block_for_drive(object_path, get_physical=True)
            if interface:
                object_pathes_whole_disk.append(interface.get_object_path())
        object_pathes_whole_disk.sort()

        return object_pathes_whole_disk

    def get_interface(self,
                      object_path: str,
                      interface_name: str) -> Union[Gio.DBusInterface, None]:
        """
        Get the interface proxy for interface_name at object_path, if Any.

        Parameters
        ----------
        object_path : str
            Object path to look up.
        interface_name : str
            D-Bus interface name to look up.

        Returns
        -------
        interface : Union[Gio.DBusInterface, None]
            A Gio.DBusInterface instance or None.

        """
        interface_name = self.__validate_interface_name(interface_name)

        interface = \
            self.__object_manager.get_interface(object_path, interface_name)
        return interface

    def get_interface_names(self, object_path: str) -> list:
        interface_names = []

        dbus_object = self.__object_manager.get_object(object_path)
        if dbus_object:
            interfaces = dbus_object.get_interfaces()
            interface_names = [interface.get_interface_name()
                               for interface in interfaces]
        return interface_names

    def get_property(self,
                     object_path: str,
                     interface_name: str,
                     property_name: str) -> Union[Any, None]:

        property_value = None
        interface_name = self.__validate_interface_name(interface_name)

        interface = \
            self.__object_manager.get_interface(object_path, interface_name)
        property_value = interface.get_property(property_name)
        if isinstance(property_value, GLib.Variant):
            property_value = property_value.unpack()

        return property_value

    @staticmethod
    def __validate_interface_name(interface_name: str) -> str:
        if interface_name:
            if interface_name in _conv_interface_names:
                interface_name = _conv_interface_names[interface_name]
            elif interface_name not in _conv_interface_names.values():
                raise ValueError(
                    "Unknown value for argument 'interface_name'.")
        return interface_name

    def get_partition_pathes(self, object_path: str) -> list[str]:
        partition_pathes = []

        interface_ptable = "org.freedesktop.UDisks2.PartitionTable"
        interface_partition = "org.freedesktop.UDisks2.Partition"

        interface = self.__object_manager.get_interface(object_path,
                                                        interface_ptable)
        if interface:
            partition_pathes = interface.get_property("partitions")
        else:
            interface = \
                self.__object_manager.get_interface(object_path,
                                                    interface_partition)
            if interface:
                object_path = interface.get_property("table")
                interface = \
                    self.__object_manager.get_interface(object_path,
                                                        interface_ptable)
                if interface:
                    partition_pathes = interface.get_property("partitions")

        return partition_pathes

    def __get_interface_filesystem(
            self, object_path: str) -> Union[UDisks.Filesystem, None]:
        interface_filesystem = None

        interface_name = "org.freedesktop.UDisks2.Encrypted"
        interface_encrypted = \
            self.__object_manager.get_interface(object_path, interface_name)
        if interface_encrypted:
            cleartext_device = \
                interface_encrypted.get_property('cleartext-device')
            if cleartext_device != '/':
                object_path = cleartext_device

        interface_name = "org.freedesktop.UDisks2.Filesystem"
        interface_filesystem = \
            self.__object_manager.get_interface(object_path, interface_name)

        return interface_filesystem

    def is_mounted(self, object_path: str) -> bool:
        mounted = False

        interface_filesystem = self.__get_interface_filesystem(object_path)
        if interface_filesystem:
            mounted = bool(interface_filesystem.get_property("mount-points"))
        return mounted

    def is_filesystem_mounted(self,
                              interface_filesystem: UDisks.Filesystem) -> bool:
        mounted = False

        if interface_filesystem:
            object_path = interface_filesystem.get_object_path()
            mounted = self.is_mounted(object_path)
        return mounted

    def get_mounted_filesystems(self,
                                object_path: str) -> list[UDisks.Filesystem]:
        mounted_filesystems = []

        partitions = self.get_partition_pathes(object_path)
        if partitions:
            for path in partitions:
                interface_filesystem = self.__get_interface_filesystem(path)
                if interface_filesystem:
                    mounted = \
                        bool(interface_filesystem.get_property("mount-points"))
                    if mounted:
                        mounted_filesystems.append(interface_filesystem)

        return mounted_filesystems

    def __get_interface_encrypted(
            self, object_path: str) -> Union[UDisks.Encrypted, None]:
        interface_encrypted = None

        interface_name = "org.freedesktop.UDisks2.Block"
        interface_block = self.__object_manager.get_interface(object_path,
                                                              interface_name)
        if interface_block:
            crypto_backing_device = \
                interface_block.get_property('crypto_backing_device')
            if crypto_backing_device != '/':
                object_path = crypto_backing_device

        interface_name = "org.freedesktop.UDisks2.Encrypted"
        interface_encrypted = \
            self.__object_manager.get_interface(object_path, interface_name)

        return interface_encrypted

    def is_locked(self, object_path: str) -> bool:
        locked = True

        interface_name = "org.freedesktop.UDisks2.Block"
        interface_block = self.__object_manager.get_interface(object_path,
                                                              interface_name)
        if interface_block:
            cleartext_block = \
                self.__client.get_cleartext_block(interface_block)
            if cleartext_block:
                interface_block = cleartext_block

            crypto_backing_device = \
                interface_block.get_property('crypto_backing_device')
            if crypto_backing_device != '/':
                locked = False

        return locked

    def get_unlocked_encrypted(
            self, object_path: str) -> Union[UDisks.Encrypted, None]:
        unlocked_encrypted = None

        locked = self.is_locked(object_path)
        if not locked:
            unlocked_encrypted = self.__get_interface_encrypted(object_path)

        return unlocked_encrypted



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'MainWindow.ui'
##
## Created by: Qt User Interface Compiler version 6.4.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QHBoxLayout, QLabel,
    QMainWindow, QMenu, QMenuBar, QProgressBar,
    QPushButton, QSizePolicy, QStatusBar, QVBoxLayout,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(500, 175)
        MainWindow.setMinimumSize(QSize(500, 0))
        self.actionAbout = QAction(MainWindow)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionAboutQt = QAction(MainWindow)
        self.actionAboutQt.setObjectName(u"actionAboutQt")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pushButton_open = QPushButton(self.centralwidget)
        self.pushButton_open.setObjectName(u"pushButton_open")

        self.horizontalLayout.addWidget(self.pushButton_open)

        self.comboBox_devices = QComboBox(self.centralwidget)
        self.comboBox_devices.setObjectName(u"comboBox_devices")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.comboBox_devices.sizePolicy().hasHeightForWidth())
        self.comboBox_devices.setSizePolicy(sizePolicy1)

        self.horizontalLayout.addWidget(self.comboBox_devices)

        self.comboBox_buffers = QComboBox(self.centralwidget)
        self.comboBox_buffers.setObjectName(u"comboBox_buffers")

        self.horizontalLayout.addWidget(self.comboBox_buffers)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.progressBar = QProgressBar(self.centralwidget)
        self.progressBar.setObjectName(u"progressBar")
        self.progressBar.setValue(0)

        self.verticalLayout.addWidget(self.progressBar)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_source = QLabel(self.centralwidget)
        self.label_source.setObjectName(u"label_source")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.label_source.sizePolicy().hasHeightForWidth())
        self.label_source.setSizePolicy(sizePolicy2)

        self.horizontalLayout_2.addWidget(self.label_source)

        self.label_srcpath = QLabel(self.centralwidget)
        self.label_srcpath.setObjectName(u"label_srcpath")
        sizePolicy1.setHeightForWidth(self.label_srcpath.sizePolicy().hasHeightForWidth())
        self.label_srcpath.setSizePolicy(sizePolicy1)
        self.label_srcpath.setScaledContents(False)
        self.label_srcpath.setWordWrap(True)
        self.label_srcpath.setMargin(6)

        self.horizontalLayout_2.addWidget(self.label_srcpath)

        self.pushButton_cancel = QPushButton(self.centralwidget)
        self.pushButton_cancel.setObjectName(u"pushButton_cancel")

        self.horizontalLayout_2.addWidget(self.pushButton_cancel)

        self.pushButton_write = QPushButton(self.centralwidget)
        self.pushButton_write.setObjectName(u"pushButton_write")

        self.horizontalLayout_2.addWidget(self.pushButton_write)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 500, 30))
        self.menubar.setLayoutDirection(Qt.RightToLeft)
        self.menuInfo = QMenu(self.menubar)
        self.menuInfo.setObjectName(u"menuInfo")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuInfo.menuAction())
        self.menuInfo.addAction(self.actionAbout)
        self.menuInfo.addAction(self.actionAboutQt)

        self.retranslateUi(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionAbout.setText(QCoreApplication.translate("MainWindow", u"About", None))
        self.actionAboutQt.setText(QCoreApplication.translate("MainWindow", u"AboutQt", None))
        self.pushButton_open.setText(QCoreApplication.translate("MainWindow", u"Select Image", None))
        self.label_source.setText(QCoreApplication.translate("MainWindow", u"Image files:", None))
        self.label_srcpath.setText(QCoreApplication.translate("MainWindow", u"-", None))
        self.pushButton_cancel.setText(QCoreApplication.translate("MainWindow", u"Cancel", None))
        self.pushButton_write.setText(QCoreApplication.translate("MainWindow", u"Write", None))
        self.menuInfo.setTitle(QCoreApplication.translate("MainWindow", u"Info", None))
    # retranslateUi

